@@ -1,7 +1,7 @@
 package Params::Sah;
 
 our $DATE = '2015-03-23'; # DATE
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 use 5.010001;
 use strict;
@@ -100,6 +100,8 @@ sub gen_validator {
         if ($opts->{on_invalid} =~ /\A(croak|carp|warn|die)\z/) {
             my $stmt = $opts->{on_invalid} =~ /\A(croak|carp)\z/ ?
                 "Carp::$opts->{on_invalid}" : $opts->{on_invalid};
+            $src .= "    undef \$_ps_res;\n" if
+                $i && $opts->{on_invalid} =~ /\A(carp|warn)\z/;
             $src .= "    $stmt(\"$data_name: \$_ps_res\") ".
                 "if !($cd->{result});\n";
         } else {
@@ -148,7 +150,7 @@ Params::Sah - Validate method/function parameters using Sah schemas
 
 =head1 VERSION
 
-This document describes version 0.02 of Params::Sah (from Perl distribution Params-Sah), released on 2015-03-23.
+This document describes version 0.03 of Params::Sah (from Perl distribution Params-Sah), released on 2015-03-23.
 
 =head1 SYNOPSIS
 
@@ -323,6 +325,54 @@ By using the Sah C<default> clause:
 
  gen_validator(['str*', default=>'green']);
 
+=head2 Why is my program failing with error message: Can't call method "state" on an undefined value?
+
+You need to use Perl 5.10 or newer and enable the 5.10 feature 'state':
+
+ use 5.010;
+
+or:
+
+ use feature 'state';
+
+=head2 How do I see the validator code being generated?
+
+Set C<$Params::Sah::DEBUG=1> before C<gen_validator()>, for example:
+
+ use Params::Sah qw(gen_validator);
+
+ $Params::Sah::DEBUG = 1;
+ gen_validator('int*', 'str');
+
+Sample output:
+
+   1|sub(\@) {
+   2|    my $_ps_args = shift;
+   3|    my $_ps_res;
+    |
+    |
+   6|    ### validating 0:
+   7|    no warnings 'void';
+   8|    my $_sahv_dpath = [];
+   9|    Carp::croak("arg0: $_ps_res") if !(    # req #0
+  10|    ((defined($_ps_args->[0])) ? 1 : (($_ps_res //= (@$_sahv_dpath ? '@'.join("/",@$_sahv_dpath).": " : "") . "Required but not specified"),0))
+    |
+  12|    &&
+    |
+  14|    # check type 'int'
+  15|    ((Scalar::Util::Numeric::isint($_ps_args->[0])) ? 1 : (($_ps_res //= (@$_sahv_dpath ? '@'.join("/",@$_sahv_dpath).": " : "") . "Not of type integer"),0)));
+    |
+    |
+  18|    ### validating 1:
+  19|    Carp::croak("arg1: $_ps_res") if !(    # skip if undef
+  20|    (!defined($_ps_args->[1]) ? 1 :
+    |
+  22|    (# check type 'str'
+  23|    ((!ref($_ps_args->[1])) ? 1 : (($_ps_res //= (@$_sahv_dpath ? '@'.join("/",@$_sahv_dpath).": " : "") . "Not of type text"),0)))));
+  24|    return;
+    |
+  26|};
+
 =head1 SEE ALSO
 
 L<Sah>, L<Data::Sah>
